#include "adwin.h"
// define NO_ADMIN_MODE to test code without an actual ADwin system connected
#ifndef NO_ADWIN_MODE
#include "ADwin_V2.h"
#endif //NO_ADWIN_MODE

#define PAR_TIMER  1
#define PAR_FBCR   2
#define PAR_RCR    3
#define PAR_ACTIVE 4
#define PAR_FCR_1  11

#define FPAR_RAMP_AMPL  1

#define DATA_FILTER_COEFFS 1
#define DATA_ACTIVE 2

ADwinInterface::ADwinInterface(QObject *parent)
    : QObject(parent), m_deviceNum(0), m_booted(false),
      m_seconds(0), m_cpuLoad(0), m_servoCount(0), m_errorNo(0),
      m_activeServo(0)
{
}

/***
 * loadFirmware
 * load the specified firmware into the ADwin hardware, which
 * must be booted (e.g. via the ADBasic program) beforehand.
 * Firmware is automatically started afterwards.
 */
bool ADwinInterface::loadFirmware(int deviceNum, QString firmware)
{
#ifndef NO_ADWIN_MODE
    // boot ADwin should happen in ADwin software for now
    // load servo firmware (stop/clean process first)
    long err;

    // make sure ADwin system is booted
    if (Test_Version(deviceNum, &err)) {
        setErrorNo(err);
        return false;
    }

    // load firmware
    QByteArray ba = firmware.toLocal8Bit();
    Stop_Process(1, deviceNum, &err);
    Load_Process(ba.data(), deviceNum, &err);
    if (err) {
        setErrorNo(err);
        return false;
    }
    Start_Process(1, deviceNum, &err);
    if (err) {
        setErrorNo(err);
        return false;
    }
#endif //NO_ADWIN_MODE

    m_deviceNum = deviceNum;
    m_booted = true;
    m_seconds = 0;
    emit connected();
    emit deviceNumChanged(m_deviceNum);
    setServoCount(8); //TODO: set this to the actual servo count, get this via a PAR

    return true;
}

/***
 * runWorker
 * this should be called periodically to update values in the GUI,
 * e.g. the seconds timer, the cpu load, and monitor values.
 */
bool ADwinInterface::runWorker()
{
    long err;
    long mon[3] = {0x8000,0x8001,0x8002};

#ifndef NO_ADWIN_MODE
    setSeconds(Get_Par(1, m_deviceNum, &err));
    setCpuLoad(Workload(0, m_deviceNum, &err));

    Set_Par(PAR_ACTIVE, m_activeServo, m_deviceNum, &err);
    GetData_Long(DATA_ACTIVE, mon, 1, 3, m_deviceNum, &err);
#else
    // just some dummy values
    setSeconds(m_seconds+1);
    setCpuLoad(85);
#endif //NO_ADWIN_MODE
    for (int ii=0; ii < 3; ++ii)
        m_monitor[ii] = mon[ii] - 0x8000;
    emit inMonitorChanged(m_monitor[0]);
    emit auxMonitorChanged(m_monitor[1]);
    emit outMonitorChanged(m_monitor[2]);
    return true;
}

/***
 * setRamp
 * Enable ramp at specified servo (will disable ramping at previous
 * servo). Set servo=0 to disable ramping.
 */
void ADwinInterface::setRamp(int servo, int freq, double ampl)
{
    long rcr = 0;
    long err = 0;
    rcr = (servo & 0xff) + ((freq & 0xff) << 8);
#ifndef NO_ADWIN_MODE
    Set_Par(PAR_RCR, rcr, m_deviceNum, &err);
    Set_FPar_Double(FPAR_RAMP_AMPL, ampl, m_deviceNum, &err);
#endif
}

/***
 * hasRamp
 * check whether a servo is ramping
 */
bool ADwinInterface::hasRamp(int servo)
{
    long err = 0;
#ifndef NO_ADWIN_MODE
    return ((Get_Par(PAR_RCR, m_deviceNum, &err) & 0xff) == servo);
#else
    return false;
#endif
}


QObject *adwin_interface_provider(QQmlEngine *engine, QJSEngine *scriptEngine)
{
    Q_UNUSED(engine)
    Q_UNUSED(scriptEngine)

    ADwinInterface *adwin = new ADwinInterface();
    return adwin;
}

